require "active_support/all"
require "bunny"
require "yaml"
require "airbrake"

require "honest_pubsub/configuration"
require "honest_pubsub/context"
require "honest_pubsub/db_logger"
require "honest_pubsub/logger"
require "honest_pubsub/logging"
require "honest_pubsub/message"
require "honest_pubsub/publisher"
require "honest_pubsub/server/rabbit_mq_subscriber"
require "honest_pubsub/version"
require "honest_pubsub/exceptions/payload_validation_error"
require 'honest_pubsub/server'
require 'honest_pubsub/subscriber'

if defined?(Rails::Railtie)
  require "honest_pubsub/middleware"
  require "honest_pubsub/railtie"
end

module HonestPubsub

  def self.root
    File.expand_path '../..', __FILE__
  end

# This method publishes payload to rabbitmq.  All listeners with appropriate
# routing keys will receive the payload.
# @param [String] routing_key Identifier of the message type
# @param [Hash] payload The data that will be passed to the subscriber

  def self.publish(routing_key, payload)
    Publisher.instance.publish(HonestPubsub::Context.instance, routing_key, payload)
  end

  def self.logger
    HonestPubsub::Logging.logger
  end

  # @param [Logger] logger Logger used by HonestPubsub
  def self.logger=(logger)
    HonestPubsub::Logging.logger = logger
  end
end
