module HonestPubsub
  class Context
    # acceptable values
    # employee - employee id that the context could have
    # user_id - ender user id of call
    # unique_id - unique identifier (could be request.uuid most of the time)
    # orig_ip_address - ip address of originating requester
    def initialize(params_hash = {})
      @data = Hashie::Mash.new(params_hash)
    end

    def user_id
      @data.user_id
    end

    def employee_id
      @data.employee_id
    end

    def as_json
      @data.as_json
    end

    def unique_id
      @data.unique_id
    end

    def originating_ip_address
      @data.orig_ip_address
    end

    def serialize
      @data.as_json
    end

    def application
      @data.application
    end

    def self.instance
      Thread.current['pubsub_context'] || {}
    end

    def self.setup_context(attrs = {})
      Thread.current['pubsub_context'] = self.new(attrs)
    end

    def self.clear!
      Thread.current['pubsub_context'] = nil
    end

    # USE AS INFREQUENTLY AS POSSIBLE.  Only use it if you absolutely must
    # as really, we want context in the object, and not the kitchen sink,
    # which is what this will turn out to be if we use regularly use this
    # mechanism.
    def method_missing(method_name, *arguments, &block)
      if @data.key?(method_name)
        @data[method_name]
      else
        super
      end

    end

    def self.from_json(data_hash)
      ::HonestPubsub::Context.new(data_hash)
    end

  end
end