# Internal log class that is used to log messages before sending, after receiving, and failure to send
module HonestPubsub
  class Logger
    def initialize(enable_publish_logging = true)
      @enabled = HonestPubsub::Configuration.configuration[:logger][:enabled]
      @enabled = enable_publish_logging if @enabled.nil?
    end

    def log_publish(routing_key, message)
      return unless @enabled
      # generate tags needed
      tags = ["PUBLISH", message[:ts], message[:pub], message[:v], routing_key]

      # FIX!!! -thl
      # Could logging like this be too slow?
      # Or should it be threaded?
      # We'll need to benchmark, as we don't want this to get too slow.
      logger.tagged(tags) { logger.warn message[:payload].as_json }

      # TODO: -thl
      # Log it into mongo as well?
    end

    def failed_publish(routing_key, properties, message)
      tags = ["FAILED_SEND", message[:ts], message[:pub], message[:v], routing_key]
      logger.tagged(tags) { logger.warn message[:payload].as_json }
    end

    def log_receive(routing_key, message)
      tags = ["RECEIVED", message[:ts], message[:pub], message[:v], routing_key, Process::pid]
      logger.tagged(tags) { logger.warn message[:payload].as_json }
    end

    def log_service(service_name, log_level, message)
      tags = ["SERVICE", service_name, Process::pid]
      log_method = log_level.to_sym.to_proc
      logger.tagged(tags) { log_method.call(logger) { message.as_json } }
    end

    def teardown
      logger.close
    end

    private
    def logger
      @logger ||= ActiveSupport::TaggedLogging.new(HonestPubsub.logger)
    end
  end
end
