
require "hashie"
require "json"

module HonestPubsub
  class Message
    @@message_version = "1"
    attr_reader :ts
    attr_reader :pub
    attr_reader :version
    attr_reader :payload
    attr_reader :context

    def initialize
    end

    # Context object should be passed into the call.
    def serialize(context, routing_key, payload)
      @ts = Time.now.to_i
      @pub = "#{routing_key}:#{Socket.gethostname()}:#{Process::pid}"
      @version = @@message_version
      @payload = payload
      @context = context
      to_hash
    end

    def parse(envelope)
      contents = Hashie::Mash.new(::JSON.parse(envelope))
      @ts = contents[:ts]
      @pub = contents[:pub]
      # Version used for messaging can get updated if we need to add extra header information, and need
      # to move each section of the library separately.
      @version = contents[:v]
      @payload = contents[:payload]
      @context = ::HonestPubsub::Context.from_json(contents[:context])
      to_hash
    end

    def to_hash
      ::Hashie::Mash.new({
        ts: @ts,
        pub: @pub,
        v: @@message_version,
        payload: @payload,
        context: @context.as_json
      })
    end

  end
end
