require 'rubygems'
require 'celluloid/autostart'
require 'celluloid/io'
require 'honest_pubsub'

module HonestPubsub
  module Server
    class ClientQueueListener
      include Celluloid

      attr_reader :worker_class

      def initialize(worker_class, request_key, queue, durable = true, topic = "honest")
        @topic        = topic
        @request_key  = request_key
        @worker_class = worker_class
        @queue_name   = queue
        @durable      = durable
        @subscriber   = ::HonestPubsub::Server::RabbitMQSubscriber.new(@request_key, @durable, @topic)
      end

      def start
        @subscriber.start(@queue_name, false) do |delivery_info, properties, envelope|
          message_received(delivery_info, properties, envelope)
          true
        end
      end

      def shutdown
        @subscriber.teardown if @subscriber.present?
        # TODO -thl
        # This the kosher thing to do in ruby?
        teardown
      end

      def message_received(delivery_info, properties, envelope)
        HonestPubsub.logger.debug("Message received by listener on #{worker_class.name} with payload: #{envelope[:payload]}")
        worker = worker_class.new(delivery_info, properties, envelope[:context])
        worker.perform!(envelope[:payload])
      rescue => e
        HonestPubsub.logger.error("Failed message for #{worker_class.name}")
        Airbrake.notify("Failed perform for #{worker_class.name}",
                        params: {
                          info:       delivery_info,
                          properties: properties,
                          context:    envelope[:context],
                          payload:    envelope[:payload] })

      end

      protected
      def teardown
      end
    end
  end
end
